/**************************************************************************

   File:                RD11++.h

   Author(s):           Robert Sim

   Created:             16 Jun 2001

   Last Revision:       $Date: 2002/01/14 20:10:15 $

   Description: 

   $Revision: 1.1.2.6 $

   $Log: RD11++.h,v $
   Revision 1.1.2.6  2002/01/14 20:10:15  simra
   Added destruction option to constructor, to destroy old robots.

   Revision 1.1.2.5  2001/11/20 20:51:49  simra
   Not sure what changed.

   Revision 1.1.2.4  2001/08/12 06:35:07  simra
   Minor mods

   Revision 1.1.2.3  2001/06/20 22:47:27  simra
   Added laser support.
   Some debugging changes to libRD11++.  Note: there is a thread safety problem.

   Revision 1.1.2.2  2001/06/19 19:26:59  simra
   Added Robot class.

   Revision 1.1.2.1  2001/06/17 14:19:28  simra
   c++ bindings for RD11.


	
   Copyright (c) 2001 by Robert Sim, Centre for Intelligent Machines,
   McGill University, Montreal, QC.  Please see the copyright notice
   included in this distribution for full details.

 **************************************************************************/

#ifndef RD11PP_H
#  define RD11PP_H


#include "sockinet.h"
#include <string>
#include <list>

namespace MMRA {
  class RD11Connection {
    iosockinet rd11socket;
    string prompt;
  public:
    RD11Connection(string host="localhost", int port=5090);
    ~RD11Connection() {};// rd11socket.close(); }
    string operator<<(const string& cmd);
    void RD11Connection::read(char* buffer, int max_size);
    void RD11Connection::write(char* buffer, int size);
    void RD11Connection::getline(char * buffer, int max_size);

  };
  
  class RangePoint {
  public:
    double r_x;
    double r_y;
    double r_theta;
    double s_range;
    double s_theta;
    
    RangePoint() : r_x(0),r_y(0),r_theta(0),s_range(0),s_theta(0) {};
    RangePoint(double _r_x, double _r_y, double _r_theta, double _s_range, 
	       double _s_theta): 
      r_x(_r_x),
      r_y(_r_y),
      r_theta(_r_theta),
      s_range(_s_range),
      s_theta(_s_theta) {};
    RangePoint(const RangePoint& r) :
      r_x(r.r_x),
      r_y(r.r_y),
      r_theta(r.r_theta),
      s_range(r.s_range),
      s_theta(r.s_theta) {};
    RangePoint& operator=(const RangePoint& r) {
      r_x=r.r_x;
      r_y=r.r_y;
      r_theta=r.r_theta;
      s_range=r.s_range;
      s_theta=r.s_theta; 
      return *this;
    }
  };

  class Pose {
  public:
    double x;
    double y;
    double theta;
    Pose() : x(0), y(0), theta(0) {};
    Pose(double _x, double _y, double _theta) :
      x(_x), y(_y), theta(_theta) {}
    Pose(const Pose& p) : x(p.x), y(p.y), theta(p.theta) {};
    Pose& operator=(const Pose& p) {
      x=p.x; y=p.y; theta=p.theta; return *this;
    }
  };

  class Robot {
    static string robot_type_names[];

    RD11Connection rd11;
    string name;
    
    // can't be copied..
    Robot(const Robot& r) {}
    Robot& operator=(const Robot& r) {return *this;}

  public:
    typedef enum {RD_NOMAD_SIMULATOR=0, RD_RWI_SIMULATOR=1,
		  RD_SCOUT_SIMULATOR=2, RD_NOMAD=3, RD_RWI=4, RD_SCOUT=5,
		  RD_CYBER=6, RD_CYBER_SIMULATOR=7, RD_POINT_SIMULATOR=8,
		  RD_GENERIC_SIMULATOR=9} Robot_Type; 
    Robot(string _name, Robot_Type _r_type, string host="localhost", int port=5090, bool destroy=false);
    ~Robot() {
      rd11<<"destroy "+name;
    }
    string operator<<(const string& cmd);
    void abs_rotate(double theta);
    void rel_rotate(double theta);
    void abs_move(double x, double y, const string& mode="");
    void rel_move(double x, double y, const string& mode="");
    void translate(double d, const string& mode="");
    list<RangePoint> sonar(const string& mode="");
    list<RangePoint> laser(const string& mode="");
    list<RangePoint> infra_red(const string& mode="");
    string get_variable(const string& var);
    void set_variable(const string& var, const string& val);
    Pose get_pose();
    void set_pose(const Pose& p);

  protected: 
    Robot_Type r_type;
    
  public:
    Robot_Type get_robot_type(void) { return r_type; }

  };

}
#endif  /* RD11++_H */
