/**************************************************************************

   File:                RD11.h

   Author(s):           Nick Roy

   Created:             22 Apr 1997

   Last Revision:       $Date: 2001/06/20 22:47:24 $

   Description: 

   $Revision: 1.2.8.2 $

   $Log: RD11.h,v $
   Revision 1.2.8.2  2001/06/20 22:47:24  simra
   Added laser support.
   Some debugging changes to libRD11++.  Note: there is a thread safety problem.

   Revision 1.2.8.1  2000/08/31 16:03:11  simra
   Generic simulator options

   Revision 1.2  1999/01/15 19:14:20  simra
   Derived Simulators from Generic_Simulator
   Removed extraneous header files
   Fixed a few compiler warnings.
   Changed robot colors.

   Revision 1.1.1.1  1999/01/05 17:28:35  simra
   First import

   Revision 1.5  1998/11/05 17:42:33  simra
   New robot types

   Revision 1.4  1997/12/17 21:12:58  ericb
   Fixed Robot_Type Enum

   Revision 1.3  1997/12/17 20:35:28  ericb
   Added RDCreate_No_Zero

   Revision 1.2  1997/06/17 20:48:42  nickr
   A version of libRD10 that works now.

 * Revision 1.1.1.1  1997/04/24  03:27:58  nickr
 * The connection library to RoboDaemon10; it slices! it dices!
 * Check one out now, and get a free set of Ginsu knives - absolutely
 * no charge! We'll even throw in the man pages!
 * And of course, in mock-Swedish:
 * Zee cunnecshun leebrery tu RubuDeemun10; it sleeces! it deeces!
 * Check oone-a oooot noo, und get a free-a set ooff Geensoo kneefes -
 * ebsulootely nu cherge-a! Ve'll ifee throo in zee mun peges!
 * Bork Bork Bork!
 *

	
   Copyright (c) 1997 by Nick Roy, Centre for Intelligent Machines,
   McGill University, Montreal, QC.  Please see the copyright notice
   included in this distribution for full details.

 **************************************************************************/

#ifndef RD11_H
#  define RD11_H

/**********************
     Include Files
 **********************/

/**********************
     Public Defines
 **********************/

/**********************
      Public Types
 **********************/

typedef struct {
  float X;
  float Y;
  float Theta;
} RD_Pose, * RD_Pose_Ptr;

typedef enum {RD_NO_ERR, RD_BAD_ARGS, RD_NO_CONNECTION, RD_SYS_ERR, 
	      RD_MEM_ERR, RD_SOCK_ERR, RD_COMMAND_ERR} RD_Err_Type;

typedef enum {RD_NOMAD_SIMULATOR=0, RD_RWI_SIMULATOR=1,
	      RD_SCOUT_SIMULATOR=2, RD_NOMAD=3, RD_RWI=4, RD_SCOUT=5,
	      RD_CYBER=6, RD_CYBER_SIMULATOR=7, RD_POINT_SIMULATOR=8,
	      RD_GENERIC_SIMULATOR=9} 
             RD_Robot_Type; 

/********************
   Public Variables 
 ********************/

#ifdef RD11_C
#  define EXTERN 
#  define INIT(x) =x
#  define ARRAYDECL(x,n) x[n]
#else
#  define EXTERN extern
#  define INIT(x)
#  define ARRAYDECL(x,n) extern x[]
#endif

EXTERN RD_Err_Type RD_Err_No;

#undef EXTERN
#undef INIT
#undef ARRAYDECL

int RD_Connect(char *Host_Name, char *Socket_Number);
int RD_Disconnect(void);
int RD_Set_Verbosity(int Verbosity);
int RD_Create(RD_Robot_Type Robot, char *Name, char *Args);
int RD_Destroy(char *Name);
int RD_Send_Command(char *Command, char *Result);
int RD_Abs_Rotate(char *Name, float Theta, float *Delta);
int RD_Rel_Rotate(char *Name, float Theta, float *Delta);
int RD_Abs_Move(char *Name, float *X, float *Y, char *Mode, float *Delta);
int RD_Rel_Move(char *Name, float *X, float *Y, char *Mode, float *Delta);
int RD_Translate(char *Name, float Distance, char *Mode, float *Delta);
int RD_Sonar(char *Name, char *Mode, float *Data_Array);
int RD_Laser(char *Name, char *Mode, float *Data_Array);
int RD_Infra_Red(char *Name, char *Mode, float *Data_Array);
int RD_Bumper(char *Name, char *Arg, float *Theta);
int RD_Get_Variable(char *Variable, char *Value);
int RD_Set_Variable(char *Variable, char *Value);
int RD_Get_Pose(char *Name, RD_Pose_Ptr Pose);
int RD_Set_Pose(char *Name, RD_Pose_Ptr Pose);
int RD_Estop (char *Name);
int RD_Create_No_Zero (RD_Robot_Type Robot, char *Name, char *Args, float x, float y, float th, float turret);

#endif  /* RD11_H */
