/* -*- C -*- */
/**************************************************************************

   File:                SocketUtilities.h

   Author(s):           Gilbert Soucy (Original version)
                        Marc Bolduc, Paul Mackenzi (mrl version)

   Created:             22 Spet 1995 (mrl version)

   Last Revision:       $Date: 2001/08/12 06:35:11 $

   Description:         Socket server/client library.

   $Revision: 1.4.8.1 $

   $Log: SocketUtilities.h,v $
   Revision 1.4.8.1  2001/08/12 06:35:11  simra
   Minor mods

   Revision 1.4  1999/11/22 21:02:01  simra
   Merged BETA1

   Revision 1.3.2.4  1999/10/14 21:26:47  simra
   REENTRANT locking routines

   Revision 1.3.2.3  1999/10/14 01:52:19  simra
   Shell hacking part deux.

   Lots of fixes to remove the dual event loops, clean up the threads and
   make libSocketUtils the preferred command handler.
   There's still a bug wrt to EOF on stdin, which I can't figure out.
   Next step is a clean gui interface.

   Revision 1.3.2.2  1999/10/13 12:17:19  simra
   Shell hacking.

   Revision 1.3.2.1  1999/10/12 23:43:58  simra
   Nick's fixes to SocketUtilities:
   Basically, I changed SockUNewSocket to allow the server to open the socket on
   either localhost or the machine, and also the server sets the SO_REUSEADDR
   option on that socket. This latter change is really quite useful if it's not
   in there -- it's been bugging me for YEARS!

   Revision 1.3  1999/04/19 16:00:01  simra
   Fixed buffer overflow in GetNextCommand
   Fixed server socket hostname lookup to specify 'localhost' for standalone
   machines.

   Revision 1.2  1999/01/05 23:38:39  simra
   Fixed SocketUtilities prototype decls

   Revision 1.1.1.1  1999/01/05 17:28:31  simra
   First import

   Revision 1.5  1996/10/17 20:11:53  bolduc
   Added sockUReturnEmptyLine as a global variables. This tells
   SockUGetNextCommand whether or not to return an empty line of no
   only and eol separator was entered.

   Also added a return_empty_string field in the Connection structure to allow
   this new feature to be enabled on a per connection basis.

   Modified SockUGetNextCommand to check the return_empty_string and return an
   empty line if this field is set and only an eol separator was entered

   Also updated the man pages about this new feature.

 * Revision 1.4  1995/10/02  18:13:22  mackenzi
 * Backward compatibility adjustments
 *
 * Revision 1.3  1995/10/02  15:45:46  mackenzi
 * Changed so that function prototypes are not included when this header file
 * is included by SocketUtilities.c; only applications using this library will
 * get the function prototypes.
 *
 * Revision 1.2  1995/09/29  22:49:47  mackenzi
 * Updated list of function prototypes and global variables.
 *
 * Revision 1.1.1.1  1995/09/22  20:29:14  bolduc
 * Initial installation of the mrl version of Gilbert Soucy's client/server
 * socket utility library.  Code documentation and man page incomplete.
 *

	
   Copyright (c) 1995 by Who AmI, Centre for Intelligent Machines,
   McGill University, Montreal, QC.  Please see the copyright notice
   included in this distribution for full details.

 **************************************************************************/

#ifndef SOCKETUTILITIES_H
#  define SOCKETUTILITIES_H

/**********************
     Include Files
 **********************/

#include <sys/time.h>
#include <stdio.h>
#ifdef _REENTRANT
#include <pthread.h>
#endif

/**********************
     Public Defines
 **********************/

#define SERVER 0
#define CLIENT 1

#define BUF_SIZE 2000
#define LINE_SIZE 2000

#ifdef SOCKU_BACK_COMPAT
# define Connection SockUConnection
# define get_next_command SockUGetNextCommand
# define get_next_connection SockUGetNextConnection
# define new_socket SockUNewSocket
# define add_connection SockUAddConnection
# define remove_connection SockURemoveConnection
# define close_all_connections SockUCloseAllConnections
# define timestamp_connection SockUTimestampConnection
# define kill_stale_connection SockUKillStaleConnections
# define socket_ofd sockUOfd
# define socket_handle_cmd sockUHandleCmd
# define socket_wait_eol sockUWaitEol
# define tz_G sockUtz
#endif /* SOCKU_BACK_COMPAT */

/**********************
      Public Types
 **********************/

typedef struct _connection {
  int fd;
  FILE *fp;
  int handle_cmd;
  int wait_eol;
  char buffer[BUF_SIZE];
  int size;
  char line[LINE_SIZE];
  int line_available;
  struct timeval last_operation;
  struct _connection *next;
  int return_empty_line;
#ifdef _REENTRANT
  pthread_mutex_t lock_m;
#endif
} SockUConnection;

/********************
   Public Variables 
 ********************/

#ifdef SOCKETUTILITIES_C
#  define EXTERN 
#  define INIT(x) =x
#  define ARRAYDECL(x,n) x[n]
#else
#  define EXTERN extern
#  define INIT(x)
#  define ARRAYDECL(x,n) x[]
#endif

/*
 *
 * To declare a global variable, use the following macro construct:
 *
 * EXTERN sockUvariable_name INIT(value);
 *
 *  Note: INIT(x) does not work for arrays or structures.  If you 
 *	  need to inititalize arrays, use the following construct:
 *
 *#ifdef SOCKETUTILITIES_C
 *  int array[3] = {1,2,3};
 *#else 
 *  extern int array[];
 *#endif 
 *
 */

/*
 * Connection initialization stuff
 *
 * Hmm, the 3 following variables control the behavior of
 * 'SockUAddConnection' and 'SockUGetNextCommand'.  In normal
 * circumstances, you can ignore them.
 * 
 * 'sockUOfd' control the opening of a FILE* for the added file descriptor.
 * It can be set to 0 or to 1.
 * 
 * 'sockUHandleCmd' tells 'SockUGetNextCommand' to actually extract or not
 * a command or to return as soon as something is available on a given
 * link.  This is determined once when adding the link with
 * 'SockUAddConnection'. Any value of 'sockUHandleCmd' after won't change
 * anything.
 * 
 * 'sockUWaitEol' tells 'SockUGetNextCommand' to read from the fd but
 * return as soon as something is read, don't wait for newline.
 *
 * 'sockUReturnEmptyLine 'tells 'SockUGetNextCommand' to return
 * an empty string when only a newline is read.
 */

#ifdef SOCKETUTILITIES_C
struct timezone sockUtz = { 4500, 0 }; /* timezone info for timestamp
						stuff (M Bolduc) */
#else
extern struct timezone sockUtz; /* timezone info for timestamp stuff
				   (Marc Bolduc) */
#endif

EXTERN int sockUOfd	          INIT(1);   /* see SOCKETUTILITIES.c */
EXTERN int sockUHandleCmd         INIT(1);
EXTERN int sockUWaitEol           INIT(1);
EXTERN int sockUReturnEmptyLine   INIT(1);

/********************
   Public Functions
 ********************/

int SockUGetNextCommand(char *cmd, int cmdsize, int *fd, FILE **fp, 
			SockUConnection **links, struct timeval *timeout);

int SockUGetNextConnection(int fdconection, SockUConnection **links, 
			   struct timeval *timeout, int verbose);

int SockUNewSocket(int *fd, char *host, char *service, int queue, 
		   int mode, int verbose);

int SockUAddConnection(SockUConnection **links, int fd);

int SockURemoveConnection(SockUConnection **links, int fd);

int SockUCloseAllConnections(SockUConnection **links);

void SockUTimestampConnection(SockUConnection *conn);
 
int SockUKillStaleConnections(SockUConnection **links, int stale_delay);

#ifdef _REENTRANT
int SockULockConnection(SockUConnection *conn);
int SockUUnlockConnection(SockUConnection *conn);
int SockUSendLockedMessage(SockUConnection *conn, char *msg);
int SockUWriteLockedMessage(SockUConnection *conn, char *msg, int len);
#endif

#undef EXTERN
#undef INIT
#undef ARRAYDECL

#endif  /* SOCKETUTILITIES_H */

